# -*- coding: utf-8 -*-

#modif. du dossier de travail
import os
os.chdir("D:/DataMining/Databases_for_mining/dataset_for_soft_dev_and_comparison/discriminant_analysis/disqual_tuto")

#librairie pandas
import pandas

#chargement de la feuille de données
#version des données à 4 variables
vote_subset = pandas.read_excel("CongressVotePipeline.xlsx",sheet_name="subset",header=0)
print(vote_subset.info())

#importation de la librairie
from fanalysis.mca import MCA

#instanciation
acm = MCA(var_labels=vote_subset.columns[:4])

#apprentissage
coord = acm.fit_transform(vote_subset.iloc[:,:4].values)

#affichage des valeurs propres
print(acm.eig_)

#valeurs propres - graphique
print(acm.plot_eigenvalues())

#coordonnées des colonnes
print(acm.col_topandas())

#nomnbre var. actives
p = vote_subset.shape[1]-1
print(p)

#calcul des fonctions de projection
import numpy
fproj = numpy.zeros(acm.col_coord_.shape)

#pour chaque colonne
for j in range(fproj.shape[1]):
    fproj[:,j] = acm.col_coord_[:,j]/(p*numpy.sqrt(acm.eig_[0,j]))

#affichage fonction
print(fproj)

#affichage plus avenant des deux premiers facteurs
print(pandas.DataFrame(fproj,index=acm.col_labels_))

#taille du tableau de données présenté à l'ADL
print(coord.shape)

#10 premières lignes
print(coord[:10,:])

#classe pour l'analyse discriminante
from sklearn.discriminant_analysis import LinearDiscriminantAnalysis

#instanciation
adl = LinearDiscriminantAnalysis()

#apprentissage
adl.fit(coord,vote_subset.group)

#affichage des coefficients des fonctions de classement
print(adl.coef_)

#la constante
print(adl.intercept_)

#reconstitution de la fonction sur indicatrices
fpValues = numpy.dot(fproj,numpy.transpose(adl.coef_))
print(fpValues)

#affichage plus joli
print(pandas.DataFrame(fpValues,index=acm.col_labels_))

#importation des données d'apprentissage
vote_train = pandas.read_excel("CongressVotePipeline.xlsx",sheet_name="train",header=0)
print(vote_train.info())

#construction du pipeline
from sklearn.pipeline import Pipeline
disqual = Pipeline([("acm",MCA(var_labels=vote_train.columns[:-1])),("adl",LinearDiscriminantAnalysis())])

#apprentissage
disqual.fit(vote_train.iloc[:,:-1].values,vote_train.group)

#accès à l'ACM, nombre de composants
print(disqual.named_steps["acm"].n_components_)

#afficher les valeurs propres
disqual.named_steps["acm"].plot_eigenvalues()

#coefs. de la fonction score
print(disqual.named_steps["adl"].coef_)

#chargement de l'échantillon test
vote_test = pandas.read_excel("CongressVotePipeline.xlsx",sheet_name="test",header=0)
print(vote_test.info())

#prediction
pVote = disqual.predict(vote_test.iloc[:,:-1].values)
print(numpy.unique(pVote,return_counts=True))

#matrice de confusion
import sklearn.metrics as metrics
print(metrics.confusion_matrix(vote_test.group,pVote))

#taux de reconnaissance
print(metrics.accuracy_score(vote_test.group,pVote))

#nombres de facteurs à tester
parametres = [{"acm__n_components":[1,2,5,10,15,20,25,30,32]}]

#initialiser un objet validation croisée
from sklearn import model_selection
cvEval = model_selection.KFold(n_splits=5,shuffle=True,random_state=1)

#outil grille de recherche
grid = model_selection.GridSearchCV(estimator=disqual,param_grid=parametres,scoring="accuracy",cv=cvEval)

#lancement de la recherche
grid.fit(vote_train.iloc[:,:-1].values,vote_train.group)

#résultats
print(pandas.DataFrame(grid.cv_results_).loc[:,["params","mean_test_score"]])

#erreur en CV
err_cv = numpy.array(pandas.DataFrame(grid.cv_results_).loc[:,["mean_test_score"]])
print(err_cv)

#librairie graphique
import matplotlib.pyplot as plt

#paramètres - nombre de facteurs testés
prm = numpy.array(parametres[0]["acm__n_components"])

#graphique
plt.plot(prm,err_cv)
plt.title("CV accuracy rate vs. factor number")
plt.ylabel("Accuracy rate")
plt.xlabel("Factor number")
plt.show()

#ecart-type de l'erreur en validation croisée
std_cv = numpy.array(pandas.DataFrame(grid.cv_results_).loc[:,["std_test_score"]])
print(std_cv)

#graphique error bar avec les intervalles de confiance
plt.errorbar(prm,err_cv,yerr=std_cv,ecolor="silver")
plt.plot(prm,err_cv-std_cv,color="silver",linestyle="dashed",linewidth=1)
plt.plot(prm,err_cv+std_cv,color="silver",linestyle="dashed",linewidth=1)
plt.show()


#bornes basses de l'intervalle de confiance
lbound = err_cv - std_cv
print(lbound)

#accuracy de référence
ref = lbound[-1]

#les facteurs qui passent
prm[numpy.where(err_cv > ref)[0]]

#graphique error bar - seuil matérialisé
plt.errorbar(prm,err_cv,yerr=std_cv,ecolor="silver")
plt.plot(prm,numpy.repeat(ref,len(prm)),color="coral",linestyle="dashed",linewidth=1)
plt.show()

#disqual avec moins d'axes
disqualReg = Pipeline([("acm",MCA(n_components=5,var_labels=vote_train.columns[:-1])),("adl",LinearDiscriminantAnalysis())])

#apprentissage
disqualReg.fit(vote_train.iloc[:,:-1].values,vote_train.group)

#prédiction
pReg = disqualReg.predict(vote_test.iloc[:,:-1].values)

#taux de reconnaissance
print(metrics.accuracy_score(vote_test.group,pReg))
